<?php

namespace EventPortal\Tests\Unit\Util;

use EventPortal\Domain\Event\Models\Event;
use EventPortal\Domain\Form\Facades\Form;
use EventPortal\Domain\Table\Facades\Table;
use EventPortal\Domain\Util\Breadcrumb;
use EventPortal\Domain\Util\Render;
use EventPortal\Tests\TestCase;
use Illuminate\Foundation\Testing\RefreshDatabase;
use Illuminate\Support\Facades\Route;
use Spatie\Snapshots\MatchesSnapshots;

class RenderTest extends TestCase
{
    use MatchesSnapshots, RefreshDatabase;

    public function setUp(): void
    {
        parent::setUp();
        $this->withoutMiddleware();

        $this->pageHeader = [
            "title" => "Dashboard",
            "description" => "A home page",
            "howToText" => 'Text that isn\'t null',
            "howToVideo" => null,
        ];

        config(['page-headers.admin.dashboard.index' => $this->pageHeader]);
    }

    /** @test */
    public function can_get_page_config(): void
    {
        $viewdata = Render::component('Admin/Dashboard')
            ->key('dashboard.index')
            ->getData();

        $this->assertMatchesJsonSnapshot(json_encode($viewdata));
    }

    /** @test */
    public function can_extend_config(): void
    {
        $viewdata = Render::component('Some/Page')
            ->key('dashboard.index')
            ->attributes(['extraData' => [
                'first' => 'Wow',
                'second' => 'noice',
            ]])->getData();

        $this->assertMatchesJsonSnapshot(json_encode($viewdata));
    }

    /** @test */
    public function can_override_title(): void
    {
        $viewdata = Render::component('Some/Page')
            ->key('dashboard.index')
            ->withTitle('Custom Title Override')
            ->getData();

        $this->assertMatchesJsonSnapshot(json_encode($viewdata));
    }

    /** @test */
    public function can_include_form(): void
    {
        $form = Form::make('event.create')->model(new Event)->render();

        $viewdata = Render::component('Some/Page')
            ->key('dashboard.index')
            ->withForm($form)
            ->getData();

        $this->assertMatchesJsonSnapshot(json_encode($viewdata));
    }

    /** @test */
    public function can_include_multiple_forms(): void
    {
        $form = Form::make('event.create')->model(new Event)->render();

        $viewdata = Render::component('Some/Page')
            ->key('dashboard.index')
            ->withForms([
                'formA' => $form,
                'formB' => $form,
            ])
            ->getData();

        $this->assertMatchesJsonSnapshot(json_encode($viewdata));
    }

    /** @test */
    public function can_include_data(): void
    {
        $viewdata = Render::component('Some/Page')
            ->key('dashboard.index')
            ->withData([
                'first' => 'baked',
                'second' => 'beans',
                'third' => [
                    [
                        'name' => 'beans',
                        'type' => 'baked',
                    ],
                    [
                        'name' => 'beans',
                        'type' => 'kidney',
                    ],
                ],
            ])
            ->getData();

        $this->assertMatchesJsonSnapshot(json_encode($viewdata));
    }

    /** @test */
    public function can_include_table(): void
    {
        factory(Event::class)->create(['name' => 'Test Event']);

        $table = Table::make('event.index')
            ->query(Event::query()->isNotArchived())
            ->request(request())
            ->get();

        $viewdata = Render::component('Some/Page')
            ->key('dashboard.index')
            ->withTable($table)
            ->getData();

        $this->assertMatchesJsonSnapshot(json_encode($viewdata));
    }

    /** @test */
    public function can_include_breadcrumbs(): void
    {
        Route::get('test')->name('test.index');
        Route::get('test-2/{id}')->name('test2.index');

        $breadcrumbs = Breadcrumb::make()
            ->add('Events', route('test.index'))
            ->add('Event 1', route('test2.index', 1))
            ->get();

        $viewdata = Render::component('Some/Page')
            ->key('dashboard.index')
            ->withBreadcrumbs($breadcrumbs)
            ->getData();


        $this->assertMatchesJsonSnapshot(json_encode($viewdata));
    }
}
